@@ -22,6 +22,7 @@ import logging
 import numbers
 import operator
 import re
+import time
 
 import iso8601
 import lz4
@@ -31,6 +32,11 @@ import six
 
 from gnocchi import utils
 
+# NOTE(sileht): pandas relies on time.strptime()
+# and often triggers http://bugs.python.org/issue7980
+# its dues to our heavy threads usage, this is the workaround
+# to ensure the module is correctly loaded before we use really it.
+time.strptime("2016-02-19", "%Y-%m-%d")
 
 LOG = logging.getLogger(__name__)
 
@@ -264,6 +270,7 @@ class BoundTimeSerie(TimeSerie):
     def _first_block_timestamp(self):
         rounded = self.round_timestamp(self.ts.index[-1],
                                        self.block_size.delta.value)
+
         return rounded - (self.block_size * self.back_window)
 
     def _truncate(self):
@@ -281,8 +288,8 @@ class AggregatedTimeSerie(TimeSerie):
 
     POINTS_PER_SPLIT = 14400
 
-    def __init__(self, sampling, ts=None, max_size=None,
-                 aggregation_method='mean'):
+    def __init__(self, sampling, aggregation_method,
+                 ts=None, max_size=None):
         """A time serie that is downsampled.
 
         Used to represent the downsampled timeserie for a single
@@ -307,11 +314,12 @@ class AggregatedTimeSerie(TimeSerie):
         self.aggregation_method = aggregation_method
 
     @classmethod
-    def from_data(cls, sampling, timestamps=None, values=None,
-                  max_size=None, aggregation_method='mean'):
-        return cls(ts=pandas.Series(values, timestamps),
-                   max_size=max_size, sampling=sampling,
-                   aggregation_method=aggregation_method)
+    def from_data(cls, sampling, aggregation_method, timestamps=None,
+                  values=None, max_size=None):
+        return cls(sampling=sampling,
+                   aggregation_method=aggregation_method,
+                   ts=pandas.Series(values, timestamps),
+                   max_size=max_size)
 
     @classmethod
     def get_split_key_datetime(cls, timestamp, sampling):
@@ -333,23 +341,18 @@ class AggregatedTimeSerie(TimeSerie):
     def split(self):
         groupby = self.ts.groupby(functools.partial(
             self.get_split_key_datetime, sampling=self.sampling))
-        keys = sorted(groupby.groups.keys())
-        for i, ts in enumerate(keys):
-            if i + 1 == len(keys):
-                yield self._split_key_to_string(ts), TimeSerie(self.ts[ts:])
-            elif i + 1 < len(keys):
-                t = self.ts[ts:keys[i + 1]]
-                del t[t.index[-1]]
-                yield self._split_key_to_string(ts), TimeSerie(t)
+        for group, ts in groupby:
+            yield self._split_key_to_string(group), TimeSerie(ts)
 
     @classmethod
-    def from_timeseries(cls, timeseries, sampling, max_size=None,
-                        aggregation_method='mean'):
+    def from_timeseries(cls, timeseries, sampling, aggregation_method,
+                        max_size=None):
         ts = pandas.Series()
         for t in timeseries:
             ts = ts.combine_first(t.ts)
-        return cls(ts=ts, sampling=sampling, max_size=max_size,
-                   aggregation_method=aggregation_method)
+        return cls(sampling=sampling,
+                   aggregation_method=aggregation_method,
+                   ts=ts, max_size=max_size)
 
     def __eq__(self, other):
         return (isinstance(other, AggregatedTimeSerie)
@@ -390,11 +393,11 @@ class AggregatedTimeSerie(TimeSerie):
                 cls._timestamps_and_values_from_dict(d['values']))
 
         return cls.from_data(
+            sampling=sampling,
+            aggregation_method=d.get('aggregation_method', 'mean'),
             timestamps=timestamps,
             values=d.get('values'),
-            max_size=d.get('max_size'),
-            sampling=sampling,
-            aggregation_method=d.get('aggregation_method', 'mean'))
+            max_size=d.get('max_size'))
 
     def to_dict(self):
         if self.ts.empty:
@@ -508,8 +511,8 @@ class AggregatedTimeSerie(TimeSerie):
         self._truncate()
 
     @staticmethod
-    def aggregated(timeseries, from_timestamp=None, to_timestamp=None,
-                   aggregation='mean', needed_percent_of_overlap=100.0):
+    def aggregated(timeseries, aggregation, from_timestamp=None,
+                   to_timestamp=None, needed_percent_of_overlap=100.0):
 
         index = ['timestamp', 'granularity']
         columns = ['timestamp', 'granularity', 'value']
@@ -630,14 +633,13 @@ class TimeSerieArchive(SerializableMixin):
         # Limit the main timeserie to a timespan mapping
         return cls(
             [AggregatedTimeSerie(
-                max_size=size,
                 sampling=sampling,
-                aggregation_method=aggregation_method)
+                aggregation_method=aggregation_method,
+                max_size=size)
              for sampling, size in definitions]
         )
 
-    def fetch(self, from_timestamp=None, to_timestamp=None,
-              timeserie_filter=None):
+    def fetch(self, from_timestamp=None, to_timestamp=None):
         """Fetch aggregated time value.
 
         Returns a sorted list of tuples (timestamp, granularity, value).
@@ -645,8 +647,6 @@ class TimeSerieArchive(SerializableMixin):
         result = []
         end_timestamp = to_timestamp
         for ts in reversed(self.agg_timeseries):
-            if timeserie_filter and not timeserie_filter(ts):
-                continue
             points = ts[from_timestamp:to_timestamp]
             try:
                 # Do not include stop timestamp
