@@ -23,13 +23,23 @@ import uuid
 import oslo_db.api
 from oslo_db import exception
 from oslo_db.sqlalchemy import enginefacade
-from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import utils as oslo_db_utils
+from oslo_log import log
+try:
+    import psycopg2
+except ImportError:
+    psycopg2 = None
+try:
+    import pymysql.constants.ER
+    import pymysql.err
+except ImportError:
+    pymysql = None
 import six
 import sqlalchemy
+from sqlalchemy.engine import url as sqlalchemy_url
+import sqlalchemy.exc
 from sqlalchemy import types
 import sqlalchemy_utils
-from stevedore import extension
 
 from gnocchi import exceptions
 from gnocchi import indexer
@@ -42,27 +52,22 @@ ArchivePolicy = base.ArchivePolicy
 ArchivePolicyRule = base.ArchivePolicyRule
 Resource = base.Resource
 ResourceHistory = base.ResourceHistory
+ResourceType = base.ResourceType
 
 _marker = indexer._marker
 
+LOG = log.getLogger(__name__)
 
-def get_resource_mappers(ext):
-    if ext.name == "generic":
-        resource_ext = ext.plugin
-        resource_history_ext = ResourceHistory
-    else:
-        tablename = getattr(ext.plugin, '__tablename__', ext.name)
-        resource_ext = type(str(ext.name),
-                            (ext.plugin, base.ResourceExtMixin, Resource),
-                            {"__tablename__": tablename})
-        resource_history_ext = type(str("%s_history" % ext.name),
-                                    (ext.plugin, base.ResourceHistoryExtMixin,
-                                     ResourceHistory),
-                                    {"__tablename__": (
-                                        "%s_history" % tablename)})
 
-    return {'resource': resource_ext,
-            'history': resource_history_ext}
+def retry_on_deadlock(f):
+    # FIXME(jd) The default values in oslo.db are useless, we need to fix that.
+    # Once it's done, let's remove that wrapper of wrapper.
+    return oslo_db.api.wrap_db_retry(retry_on_deadlock=True,
+                                     retry_on_request=True,
+                                     max_retries=10,
+                                     retry_interval=0.1,
+                                     inc_retry_interval=True,
+                                     max_retry_interval=2)(f)
 
 
 class PerInstanceFacade(object):
@@ -103,14 +108,140 @@ class PerInstanceFacade(object):
             self.trans._factory._writer_engine.dispose()
 
 
-class SQLAlchemyIndexer(indexer.IndexerDriver):
-    resources = extension.ExtensionManager('gnocchi.indexer.resources')
+class ResourceClassMapper(object):
+    def __init__(self):
+        self._cache = {'generic': {'resource': base.Resource,
+                                   'history': base.ResourceHistory}}
+        self._lock = threading.RLock()
 
-    _RESOURCE_CLASS_MAPPER = {ext.name: get_resource_mappers(ext)
-                              for ext in resources.extensions}
+    @staticmethod
+    def _build_class_mappers(resource_type, baseclass=None):
+        tablename = resource_type.tablename
+        # TODO(sileht): Add columns
+        if not baseclass:
+            baseclass = resource_type.to_baseclass()
+        resource_ext = type(
+            str("%s_resource" % tablename),
+            (baseclass, base.ResourceExtMixin, base.Resource),
+            {"__tablename__": tablename})
+        resource_history_ext = type(
+            str("%s_history" % tablename),
+            (baseclass, base.ResourceHistoryExtMixin, base.ResourceHistory),
+            {"__tablename__": ("%s_history" % tablename)})
+        return {'resource': resource_ext,
+                'history': resource_history_ext}
+
+    def get_classes(self, resource_type):
+        # NOTE(sileht): Most of the times we can bypass the lock so do it
+        try:
+            return self._cache[resource_type.tablename]
+        except KeyError:
+            pass
+        # TODO(sileht): if the table doesn't exis
+        with self._lock:
+            try:
+                return self._cache[resource_type.tablename]
+            except KeyError:
+                mapper = self._build_class_mappers(resource_type)
+                self._cache[resource_type.tablename] = mapper
+                return mapper
+
+    @retry_on_deadlock
+    def map_and_create_tables(self, resource_type, facade):
+        with self._lock:
+            # NOTE(sileht): map this resource_type to have
+            # Base.metadata filled with sa.Table objects
+            mappers = self.get_classes(resource_type)
+            tables = [Base.metadata.tables[klass.__tablename__]
+                      for klass in mappers.values()]
+            try:
+                with facade.writer_connection() as connection:
+                    Base.metadata.create_all(connection, tables=tables)
+            except exception.DBError as e:
+                # HACK(jd) Sometimes, PostgreSQL raises an error such as
+                # "current transaction is aborted, commands ignored until end
+                # of transaction block" on its own catalog, so we need to
+                # retry, but this is not caught by oslo.db as a deadlock. This
+                # is likely because when we use Base.metadata.create_all(),
+                # sqlalchemy itself gets an error it does not catch or
+                # something. So this is paperover I guess.
+                inn_e = e.inner_exception
+                if (psycopg2
+                   and isinstance(inn_e, sqlalchemy.exc.InternalError)
+                   and isinstance(inn_e.orig, psycopg2.InternalError)
+                    # current transaction is aborted
+                   and inn_e.orig.pgcode == '25P02'):
+                    raise exception.RetryRequest(e)
+                raise
+
+    def unmap_and_delete_tables(self, resource_type, connection):
+        with self._lock:
+            # NOTE(sileht): map this resource_type to have
+            # Base.metadata filled with sa.Table objects
+            mappers = self.get_classes(resource_type)
+            tables = [Base.metadata.tables[klass.__tablename__]
+                      for klass in mappers.values()]
+
+            if connection is not None:
+                # NOTE(sileht): Base.metadata.drop_all doesn't
+                # issue CASCADE stuffs correctly at least on postgresql
+                # We drop foreign keys manually to not lock the destination
+                # table for too long during drop table.
+                # It's safe to not use a transaction since
+                # the resource_type table is already cleaned and commited
+                # so this code cannot be triggerred anymore for this
+                # resource_type
+                for table in tables:
+                    for fk in table.foreign_key_constraints:
+                        self._safe_execute(
+                            connection,
+                            sqlalchemy.schema.DropConstraint(fk))
+                for table in tables:
+                    self._safe_execute(connection,
+                                       sqlalchemy.schema.DropTable(table))
+
+            # TODO(sileht): Remove this resource on other workers
+            # by using expiration on cache ?
+            for table in tables:
+                Base.metadata.remove(table)
+            del self._cache[resource_type.tablename]
+
+    @retry_on_deadlock
+    def _safe_execute(self, connection, works):
+        # NOTE(sileht): we create a transaction to ensure mysql
+        # create locks on other transaction...
+        trans = connection.begin()
+        connection.execute(works)
+        trans.commit()
+
+
+class SQLAlchemyIndexer(indexer.IndexerDriver):
+    _RESOURCE_TYPE_MANAGER = ResourceClassMapper()
+
+    @classmethod
+    def _create_new_database(cls, url):
+        """Used by testing to create a new database."""
+        purl = sqlalchemy_url.make_url(
+            cls.dress_url(
+                url))
+        purl.database = purl.database + str(uuid.uuid4()).replace('-', '')
+        new_url = str(purl)
+        sqlalchemy_utils.create_database(new_url)
+        return new_url
+
+    @staticmethod
+    def dress_url(url):
+        # If no explicit driver has been set, we default to pymysql
+        if url.startswith("mysql://"):
+            url = sqlalchemy_url.make_url(url)
+            url.drivername = "mysql+pymysql"
+            return str(url)
+        return url
 
     def __init__(self, conf):
-        conf.set_override("connection", conf.indexer.url, "database")
+        conf.set_override("connection",
+                          self.dress_url(conf.indexer.url),
+                          "database")
         self.conf = conf
         self.facade = PerInstanceFacade(conf)
 
@@ -129,7 +260,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def get_engine(self):
         return self.facade.get_engine()
 
-    def upgrade(self, nocreate=False):
+    def upgrade(self, nocreate=False, create_legacy_resource_types=False):
         from alembic import command
         from alembic import migration
 
@@ -147,10 +278,112 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 else:
                     command.upgrade(cfg, "head")
 
-    def _resource_type_to_class(self, resource_type, purpose="resource"):
-        if resource_type not in self._RESOURCE_CLASS_MAPPER:
-            raise indexer.NoSuchResourceType(resource_type)
-        return self._RESOURCE_CLASS_MAPPER[resource_type][purpose]
+        # TODO(sileht): generic shouldn't be a particular case
+        # we must create a rt_generic and rt_generic_history table
+        # like other type
+        for rt in base.get_legacy_resource_types():
+            if not (rt.name == "generic" or create_legacy_resource_types):
+                continue
+            try:
+                with self.facade.writer() as session:
+                    session.add(rt)
+            except exception.DBDuplicateEntry:
+                pass
+            self._RESOURCE_TYPE_MANAGER.map_and_create_tables(rt, self.facade)
+
+    # NOTE(jd) We can have deadlock errors either here or later in
+    # map_and_create_tables(). We can't decorate create_resource_type()
+    # directly or each part might retry later on its own and cause a
+    # duplicate. And it seems there's no way to use the same session for
+    # both adding the resource_type in our table and calling
+    # map_and_create_tables() :-(
+    @retry_on_deadlock
+    def _add_resource_type(self, resource_type):
+        try:
+            with self.facade.writer() as session:
+                session.add(resource_type)
+        except exception.DBDuplicateEntry:
+            raise indexer.ResourceTypeAlreadyExists(resource_type.name)
+
+    def create_resource_type(self, resource_type):
+        # NOTE(sileht): mysql have a stupid and small length limitation on the
+        # foreign key and index name, so we can't use the resource type name as
+        # tablename, the limit is 64. The longest name we have is
+        # fk_<tablename>_history_revision_resource_history_revision,
+        # so 64 - 46 = 18
+        tablename = "rt_%s" % uuid.uuid4().hex[:15]
+        resource_type = ResourceType(name=resource_type.name,
+                                     tablename=tablename,
+                                     attributes=resource_type.attributes)
+
+        # NOTE(sileht): ensure the driver is able to store the request
+        # resource_type
+        resource_type.to_baseclass()
+
+        self._add_resource_type(resource_type)
+
+        self._RESOURCE_TYPE_MANAGER.map_and_create_tables(resource_type,
+                                                          self.facade)
+
+        return resource_type
+
+    def get_resource_type(self, name):
+        with self.facade.independent_reader() as session:
+            return self._get_resource_type(session, name)
+
+    def _get_resource_type(self, session, name):
+        resource_type = session.query(ResourceType).get(name)
+        if not resource_type:
+            raise indexer.NoSuchResourceType(name)
+        return resource_type
+
+    @staticmethod
+    def get_resource_type_schema():
+        return base.RESOURCE_TYPE_SCHEMA_MANAGER
+
+    @staticmethod
+    def get_resource_attributes_schemas():
+        return [ext.plugin.schema() for ext in ResourceType.RESOURCE_SCHEMAS]
+
+    def list_resource_types(self):
+        with self.facade.independent_reader() as session:
+            return list(session.query(ResourceType).order_by(
+                ResourceType.name.asc()).all())
+
+    # NOTE(jd) We can have deadlock errors either here or later in
+    # map_and_create_tables(). We can't decorate delete_resource_type()
+    # directly or each part might retry later on its own and cause a
+    # duplicate. And it seems there's no way to use the same session for
+    # both adding the resource_type in our table and calling
+    # map_and_create_tables() :-(
+    @retry_on_deadlock
+    def _delete_resource_type(self, name):
+        try:
+            with self.facade.writer() as session:
+                resource_type = self._get_resource_type(session, name)
+                session.delete(resource_type)
+        except exception.DBReferenceError as e:
+            if (e.constraint in [
+                    'fk_resource_resource_type_name',
+                    'fk_resource_history_resource_type_name',
+                    'fk_rh_resource_type_name']):
+                raise indexer.ResourceTypeInUse(name)
+            raise
+        return resource_type
+
+    def delete_resource_type(self, name):
+        if name == "generic":
+            raise indexer.ResourceTypeInUse(name)
+
+        resource_type = self._delete_resource_type(name)
+
+        with self.facade.writer_connection() as connection:
+            self._RESOURCE_TYPE_MANAGER.unmap_and_delete_tables(resource_type,
+                                                                connection)
+
+    def _resource_type_to_classes(self, session, name):
+        resource_type = self._get_resource_type(session, name)
+        return self._RESOURCE_TYPE_MANAGER.get_classes(resource_type)
 
     def list_archive_policies(self):
         with self.facade.independent_reader() as session:
@@ -215,6 +448,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             raise indexer.ArchivePolicyRuleAlreadyExists(name)
         return apr
 
+    @retry_on_deadlock
     def create_metric(self, id, created_by_user_id, created_by_project_id,
                       archive_policy_name,
                       name=None, resource_id=None):
@@ -234,9 +468,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             raise
         return m
 
+    @retry_on_deadlock
     def list_metrics(self, names=None, ids=None, details=False,
                      status='active', **kwargs):
-        if ids == []:
+        if ids is not None and not ids:
             return []
         with self.facade.independent_reader() as session:
             q = session.query(Metric).filter(
@@ -252,18 +487,20 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
             return list(q.all())
 
+    @retry_on_deadlock
     def create_resource(self, resource_type, id,
                         created_by_user_id, created_by_project_id,
                         user_id=None, project_id=None,
                         started_at=None, ended_at=None, metrics=None,
                         **kwargs):
-        resource_cls = self._resource_type_to_class(resource_type)
         if (started_at is not None
            and ended_at is not None
            and started_at > ended_at):
             raise ValueError(
                 "Start timestamp cannot be after end timestamp")
         with self.facade.writer() as session:
+            resource_cls = self._resource_type_to_classes(
+                session, resource_type)['resource']
             r = resource_cls(
                 id=id,
                 type=resource_type,
@@ -291,16 +528,17 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
             return r
 
-    @oslo_db.api.retry_on_deadlock
+    @retry_on_deadlock
     def update_resource(self, resource_type,
                         resource_id, ended_at=_marker, metrics=_marker,
                         append_metrics=False,
                         create_revision=True,
                         **kwargs):
-        resource_cls = self._resource_type_to_class(resource_type)
-        resource_history_cls = self._resource_type_to_class(resource_type,
-                                                            "history")
         with self.facade.writer() as session:
+            classes = self._resource_type_to_classes(session, resource_type)
+            resource_cls = classes["resource"]
+            resource_history_cls = classes["history"]
+
             try:
                 # NOTE(sileht): We use FOR UPDATE that is not galera friendly,
                 # but they are no other way to cleanly patch a resource and
@@ -400,6 +638,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         session.expire(r, ['metrics'])
 
+    @retry_on_deadlock
     def delete_resource(self, resource_id):
         with self.facade.writer() as session:
             # We are going to delete the resource; the on delete will set the
@@ -412,9 +651,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     Resource.id == resource_id).delete() == 0:
                 raise indexer.NoSuchResource(resource_id)
 
+    @retry_on_deadlock
     def get_resource(self, resource_type, resource_id, with_metrics=False):
-        resource_cls = self._resource_type_to_class(resource_type)
         with self.facade.independent_reader() as session:
+            resource_cls = self._resource_type_to_classes(
+                session, resource_type)['resource']
             q = session.query(
                 resource_cls).filter(
                     resource_cls.id == resource_id)
@@ -422,9 +663,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 q = q.options(sqlalchemy.orm.joinedload('metrics'))
             return q.first()
 
-    def _get_history_result_mapper(self, resource_type):
-        resource_cls = self._resource_type_to_class(resource_type)
-        history_cls = self._resource_type_to_class(resource_type, 'history')
+    def _get_history_result_mapper(self, session, resource_type):
+        classes = self._resource_type_to_classes(session, resource_type)
+        resource_cls = classes['resource']
+        history_cls = classes['history']
 
         resource_cols = {}
         history_cols = {}
@@ -446,7 +688,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         class Result(base.ResourceJsonifier, base.GnocchiBase):
             def __iter__(self):
-                return models.ModelIterator(self, iter(stmt.c.keys()))
+                return iter((key, getattr(self, key)) for key in stmt.c.keys())
 
         sqlalchemy.orm.mapper(
             Result, stmt, primary_key=[stmt.c.id, stmt.c.revision],
@@ -461,6 +703,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         return Result
 
+    @retry_on_deadlock
     def list_resources(self, resource_type='generic',
                        attribute_filter=None,
                        details=False,
@@ -470,12 +713,14 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                        sorts=None):
         sorts = sorts or []
 
-        if history:
-            target_cls = self._get_history_result_mapper(resource_type)
-        else:
-            target_cls = self._resource_type_to_class(resource_type)
-
         with self.facade.independent_reader() as session:
+            if history:
+                target_cls = self._get_history_result_mapper(
+                    session, resource_type)
+            else:
+                target_cls = self._resource_type_to_classes(
+                    session, resource_type)["resource"]
+
             q = session.query(target_cls)
 
             if attribute_filter:
@@ -539,18 +784,37 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                         all_resources.extend(resources)
                     else:
                         if is_history:
-                            target_cls = self._resource_type_to_class(
-                                type, "history")
+                            target_cls = self._resource_type_to_classes(
+                                session, type)['history']
                             f = target_cls.revision.in_(
                                 [r.revision for r in resources])
                         else:
-                            target_cls = self._resource_type_to_class(type)
+                            target_cls = self._resource_type_to_classes(
+                                session, type)["resource"]
                             f = target_cls.id.in_([r.id for r in resources])
 
                         q = session.query(target_cls).filter(f)
                         # Always include metrics
                         q = q.options(sqlalchemy.orm.joinedload('metrics'))
-                        all_resources.extend(q.all())
+                        try:
+                            all_resources.extend(q.all())
+                        except sqlalchemy.exc.ProgrammingError as e:
+                            # NOTE(jd) This exception can happen when the
+                            # resources and their resource type have been
+                            # deleted in the meantime:
+                            #  sqlalchemy.exc.ProgrammingError:
+                            #    (pymysql.err.ProgrammingError)
+                            #    (1146, "Table \'test.rt_f00\' doesn\'t exist")
+                            # In that case, just ignore those resources.
+                            if (not pymysql
+                               or not isinstance(
+                                   e, sqlalchemy.exc.ProgrammingError)
+                               or not isinstance(
+                                   e.orig, pymysql.err.ProgrammingError)
+                               or (e.orig.args[0]
+                                   != pymysql.constants.ER.NO_SUCH_TABLE)):
+                                raise
+
             return all_resources
 
     def expunge_metric(self, id):
@@ -561,7 +825,8 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def delete_metric(self, id):
         with self.facade.writer() as session:
             if session.query(Metric).filter(
-                    Metric.id == id).update({"status": "delete"}) == 0:
+                Metric.id == id, Metric.status == 'active').update(
+                    {"status": "delete"}) == 0:
                 raise indexer.NoSuchMetric(id)
 
 
