@@ -138,6 +138,12 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
                               indexer=self.index,
                               auth=self.auth)
 
+    @staticmethod
+    def runTest():
+        pass
+
+
+class RootTest(RestTest):
     def test_deserialize_force_json(self):
         with self.app.use_admin_user():
             self.app.post(
@@ -170,10 +176,6 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
         self.assertIs(type(status['storage']['summary']['metrics']), int)
         self.assertIs(type(status['storage']['summary']['measures']), int)
 
-    @staticmethod
-    def runTest():
-        pass
-
 
 class ArchivePolicyTest(RestTest):
     """Test the ArchivePolicies REST API.
@@ -322,11 +324,6 @@ class MetricTest(RestTest):
                 status=403)
 
     def test_add_measures_back_window(self):
-        if self.conf.storage.driver == 'influxdb':
-            # FIXME(sileht): Won't pass with influxdb because it doesn't
-            # check archive policy
-            raise testcase.TestSkipped("InfluxDB issue")
-
         ap_name = str(uuid.uuid4())
         with self.app.use_admin_user():
             self.app.post_json(
@@ -755,6 +752,41 @@ class ResourceTest(RestTest):
                 "ended_at": "2014-01-03T02:02:02+00:00",
             },
             resource_type='image')),
+        ('host', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host')),
+        ('host_disk', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+                "device_name": "test-device"
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host_disk')),
+        ('host_network_interface', dict(
+            attributes={
+                "started_at": "2014-01-03T02:02:02+00:00",
+                "user_id": str(uuid.uuid4()),
+                "project_id": str(uuid.uuid4()),
+                "host_name": "test-host",
+                "device_name": "test-device"
+            },
+            patchable_attributes={
+                "ended_at": "2014-01-03T02:02:02+00:00",
+            },
+            resource_type='host_network_interface')),
     ]
 
     @classmethod
